%% /u/sy/beebe/src/bwkawk/20020210.profile/README.PROFILE, Sat Mar 16 08:18:09 2002
%% Edit by Nelson H. F. Beebe <beebe@math.utah.edu>

============
INTRODUCTION
============

This directory contains an extension of the AT&T/Lucent Bell
Laboratories awk implementation to support line-based run-time
profiling for each code line in user programs written in awk.

The new feature for 2002 is the incorporation of access to a
high-resolution timer on Sun Solaris 2.7 (or later) UltraSPARC
systems.  The earlier release of the profiler included such timers for
IBM RS/6000 AIX and Compaq/DEC Alpha OSF/1 systems.  On all other
systems, the profiler uses the Standard C clock() function, which is
likely to be too coarse on modern systems to provide useful data.




=============================================
UPDATING THE PROFILER AFTER A NEW AWK RELEASE
=============================================

In the interests of making it possible to transfer these extensions to
newer versions of awk that Brian Kernighan releases from time to time
at

	http://cm.bell-labs.com/who/bwk/awk.tar.gz

These releases only rarely introduce new features; most changes are
very minor ones to fix extremely rare and obscure bugs, or to enhance
the code to be compilable on more systems, with more compilers, such
as the changes that I proposed to Brian to permit builds with C++
compilers.  awk is an exemplary software package that is extremely
clean, highly portable, and supremely reliable.

I have adopted a layering procedure for my profiling extensions to
strictly minimize changes to the original awk code.

(1) In an empty directory, unbundle the awk distribution from the
    above source, then copy in these new files from a previous release
    of the profiling extensions:

		Makefile
		README.PROFILE
		TIMER.NOTES
		args.h
		config.h
		cpuclock.c
		cpuclock.daa
		cpuclock.h
		cpuclock.s
		fee2.c
		fee3.c
		fee4.c
		globtest.awk
		mail.txt
		patch.pro
		profile.c
		profile.h
		suntime.c
		test.dat
		timercheck.c
		timercheck.s

    NONE of them exist in a standard awk distribution.

(2) Do

	cp awk.1 pawk.1
	mv makefile makefile.org

    The makefile.org file is required by Makefile.

(3) Use patch to apply patch.pro to attempt to automatically update
    just three awk source files, based on manual updates made for an
    earlier awk release, first saving the original files under RCS
    control:

	mkdir RCS
	ci -l -t-'Original version.' lib.c main.c run.c pawk.1
	patch < patch.pro

    At least one patch might fail: the setting of the version string
    in main.c.  Patch that manually if necessary.

    Apart from these automatically-applied patches, NO CHANGES
    WHATSOEVER are required in any of the original awk distribution
    files.

(4) Check that the setting of VERSION in Makefile matches that in the
    version string in main.c: it should be an ISO date of the form
    20020210 (== 10-Feb-2002).

(5) Build and install the profiling version with the usual GNU-style
    recipe:

	./configure && make all check install

    The Makefile contains leading comments on how to change the name
    of installed programs, how to alter compilation options and
    installation locations, and how to make an unmodified, and thus
    unprofiled, awk.




=====================
UPDATE ON TIMER.NOTES
=====================

On Thu, 14 Mar 2002 05:49:55 -0700 (MST), I responded to a query on
the reliable_computing list for help in accurate timing of code
fragments, and in my response, included a copy of TIMER.NOTES, dated
7-Oct-1996.

My posting was followed on Fri, 15 Mar 2002 10:47:44 +1100 (EST) by a
response from Richard Smith, a Sun engineer in Melbourne, Australia,
noting that the obnoxious restriction on access to the high-resolution
timer was lifted in Sun Solaris 2.7, which came out long after my 1996
experiments.  Until Richard's letter, I was unaware of that new timer,
and I'm extremely pleased to learn of its availability.

Richard Smith shows in his letter how to get this timing code inlined;
I elected not to do that, for these reasons:

	(1) There is additional work going on anyway to record the
	    profiling data.

	(2) Inlining would thus be unlikely to be of significance.

	(3) Because the inline code only works on a single CPU system,
	    whereas I routinely work on multiple-CPU Sun systems where
	    processes can migrate between CPUs in different time
	    slices.  The Sun gethrtime() function handles this
	    migration transparently, whereas direct access to the
	    UltraSPARC high-resolution clock would not.

Our correspondence is appended to mail.txt, which also records a few
earlier exchanges between me and Brian Kernighan from 1996.

I therefore decided to take the opportunity to update the profiling
code to incorporate access to this much-improved timer, and also to
layer it over the very latest awk release (currently, 10-Feb-2002).
These changes introduced very small changes to cpuclock.c to add
conditionals for defined(__sparc) to get the better timer, and to
profile.c, to document the timer data type size in the profiles, and
to add an extra report column containing the cumulative percent of
clock ticks, making it easier for the human profiler to decide where
to abandon further optimization effort.




===============
NEW MANUAL PAGE
===============

With this 2002 release, I took the liberty of preparing a manual page,
pawk.1, for the profiling awk, by making very small changes to awk.1,
plus the addition of a new section on how to get profile report.
