/* From Sun Solaris 2.8 "man 3C gethrtime" */

/***********************************************************************
Date: Fri, 15 Mar 2002 10:47:44 +1100 (EST)
From: Richard Smith - Systems Engineer - Melbourne <Richard.Smith@Sun.COM>
Reply-To: Richard Smith - Systems Engineer - Melbourne <Richard.Smith@Sun.COM>
Subject: Re: Fortran 90/95-timings
To: reliable_computing@interval.louisiana.edu
MIME-Version: 1.0
Content-Type: TEXT/plain; charset=us-ascii
Content-MD5: jSDuc3Aonvn1JFX0DEK5MQ==
X-Mailer: dtmail 1.3.0 @(#)CDE Version 1.4.2 SunOS 5.8 sun4u sparc 
Sender: owner-reliable_computing@interval.louisiana.edu
Precedence: bulk

Following up on Nelson Beebe's comments, SPARC does indeed have a high
resolution clock that increments every cpu clock cycle. In the SPARC
architecture rdtick causes a privileged_action exception if PSTATE.PRIV = 0
and TICK.NPT = 1. From Solaris 7 onwards the default setting of the bits
permits rdtick in user code.

An alternative is to use the high resolution timer routine gethrtime(3C).
Admittedly it has higher overhead than rdtick but will work when threads get
migrated across multiple cpus. Depending on which method you use and the
specific cpu you're running on overhead is in the region of 10-100 cycles.
At a minimum rdtick stalls the cpu pipeline.

I tend to use inline(1) assembler code fragments to instrument code. Infact
it is possible to use a fragment to replace gethrtime with rdtick. Note the
different argument conventions required between v8 and v9:

gethrtimev8plus.il:
.inline gethrtime,0
        rd      %tick,%o0
        sllx    %o0,32,%o1
        srlx    %o0,32,%o0
        srlx    %o1,32,%o1
.end

gethrtimev9.il:
.inline gethrtime,0
        rd     %tick,%o0
.end

cc -fast gethrtimev8plus.il junk.c ...

Fortran code should use a pragma to specify that the "function" has a C
interface:

!$pragma C(gethrtime)

Related tricks can be used to exploit the hardware performance counters.
More information can be found in "Techniques For Optimizing Applications" by
Garg and Sharapov.

For Intel architecture I suggest consulting "Michael Abrash's Graphics
Programming Black Book" Ch 3 Zen Timer
http://www.ddj.com/documents/s=865/ddj0165f/

============================================================================
   ,-_|\   Richard Smith - SE Melbourne
  /     \  Sun Microsystems Australia         Phone : +61 3 9869 6200
richard.smith@Sun.COM                        Direct : +61 3 9869 6224  
  \_,-._/  476 St Kilda Road                    Fax : +61 3 9869 6290
       v   Melbourne Vic 3004 Australia
===========================================================================

***********************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

int main(void)
{
    hrtime_t start, end;
    int i, iters = 1000;

    start = gethrtime();
    for (i = 0; i < iters; i++)
	    getpid();
    end = gethrtime();

    printf("Avg getpid() time = %lld nsec\n", (end - start) / iters);

    return (EXIT_SUCCESS);
}
